package edu.mit.ll.group43.surfaceoptimization;

import java.util.ArrayList;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;
import edu.mit.ll.group43.surfaceoptimization.aircraft.MinimumSeparationCalculator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.WeightClass;

/**
 * Implementations of this abstract class are responsible for optimally
 * re-sequencing the FCFS sequence they are given, with respect to
 * minimum separation requirements and CPS.
 * 
 * @author William Hawkins
 *
 */
public abstract class Sequencer {

	/**
	 * Constrained Position Shifting value
	 */
	protected int CPS;
	protected MinimumSeparationCalculator<? extends WeightClass> CALC;
	
	/**
	 * Constructor
	 * 
	 * @param cps
	 */
	public Sequencer(int cps, MinimumSeparationCalculator<? extends WeightClass> calc) {
		CPS = cps;
		CALC = calc;
	}
	
	/**
	 * Re-sequences the given FCFS sequence with respect to minimum
	 * separation requirements and CPS.
	 * 
	 * @param seq
	 * @return
	 */
	public abstract ArrayList<Aircraft> sequence(ArrayList<Aircraft> seq);
}
