package edu.mit.ll.group43.surfaceoptimization.dp.factory;

import edu.mit.ll.group43.surfaceoptimization.aircraft.MinimumSeparationCalculator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.WeightClass;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.cost.AllGapsSpotCostStrategy;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.cost.CostStrategy;

/**
 * This factory is used to have the DPSequencer carry out the All Gaps
 * algorithm, which allows the aircraft currently being sequenced to
 * look back at all predicted takeoff times and determine if there are
 * any gaps it can fit into without violating CPS, ready times, or adding
 * delay to the second of the two aircraft it is trying to fit between.
 * 
 * @author William Hawkins
 *
 */
public class AllGapsSpotDPSequencerFactory extends SimpleSpotDPSequencerFactory {

	public AllGapsSpotDPSequencerFactory(int cps,
			MinimumSeparationCalculator<? extends WeightClass> calc) {
		super(cps, calc);
	}

	@Override
	public CostStrategy getCostStrategy() {
		return new AllGapsSpotCostStrategy(cps, calc);
	}
}
