package edu.mit.ll.group43.surfaceoptimization.dp.factory.cost;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;
import edu.mit.ll.group43.surfaceoptimization.aircraft.MinimumSeparationCalculator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.WeightClass;
import edu.mit.ll.group43.surfaceoptimization.network.NetworkEdge;
import edu.mit.ll.group43.surfaceoptimization.network.dp.CPSNetworkNode;

/**
 * Solves for the shortest path in the CPS network by combining the
 * One Gap Force and All Gaps algorithms.
 * 
 * @author William Hawkins
 *
 */
public class AGOGFSpotCostStrategy extends OneGapForceSpotCostStrategy {

	private SmartAllGapsSpotCostStrategy allGaps;
	
	public AGOGFSpotCostStrategy(int cps,
			MinimumSeparationCalculator<? extends WeightClass> calc) {
		super(cps, calc);
		allGaps = new SmartAllGapsSpotCostStrategy(cps, calc);
	}
	
	/**
	 * Returns the additional delay that would be incurred if the optimal spot
	 * release time of the last aircraft in the target node was calculated
	 * immediately after that of the source node in the optimization. This will
	 * be the lower of the two calculated using the All Gaps and One Gap Force
	 * heuristics.
	 * 
	 * @param edge
	 */
	@Override
	public int getEdgeWeight(NetworkEdge edge) {
		
		// Get the weights associated with One Gap Force and All Gaps
		Aircraft follower = ((CPSNetworkNode) edge.getTarget()).getLastAircraft();
		int weight = super.getEdgeWeight(edge);
		int agWeight = allGaps.earliestRelease(edge, sortedAlphasSoFar(edge)) - follower.getSpotReadyTime();
		
		// Take the better of the two
		if (agWeight < weight) {
			optSpotReleases.put((CPSNetworkNode) edge.getSource(), agWeight + follower.getSpotReadyTime());
			weight = agWeight;
		}
		
		return weight;
	}

}
