package edu.mit.ll.group43.surfaceoptimization.dp.factory.cost;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;
import edu.mit.ll.group43.surfaceoptimization.aircraft.MinimumSeparationCalculator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.WeightClass;
import edu.mit.ll.group43.surfaceoptimization.network.dp.CPSNetworkNode;
import edu.mit.ll.group43.surfaceoptimization.network.dp.SmartCPSNetworkNode;

/**
 * Spot cost strategy for All Gaps that considers an entire spot release sequence
 * when determining the predecessor bound rather than just the optimal spot release
 * time of the last released aircraft. This is important when All Gaps is used as
 * part of the All Gaps / One Gap Force (AG/OGF) algorithm, which is the only place
 * this variant of All Gaps is used.
 * 
 * @author William Hawkins
 *
 */
public class SmartAllGapsSpotCostStrategy extends AllGapsSpotCostStrategy {
	
	public SmartAllGapsSpotCostStrategy(int cps,
			MinimumSeparationCalculator<? extends WeightClass> calc) {
		super(cps, calc);
	}
	
	/**
	 * Computes the earliest time at which current can be released, with respect
	 * to its release sequence predecessors.
	 * 
	 * WARNING: Should always be compared with current's spot ready time since
	 * this method will return zero if none of the aircraft already placed in
	 * the release sequence can follow current.
	 * 
	 * @param prevNode
	 * @param current
	 * @return
	 */
	@Override
	public int safePrevTimeBound(CPSNetworkNode prevNode, Aircraft current) {
		return CostStrategyUtils.safePrevTimeBound(((SmartCPSNetworkNode) prevNode).getReleaseSequence(),
				current, CPS);
	}

}
