package edu.mit.ll.group43.surfaceoptimization.input;

import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collections;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;
import edu.mit.ll.group43.surfaceoptimization.aircraft.AircraftSpotReadyComparator;

/**
 * Responsible for reading in the First Come First Serve (FCFS) aircraft sequence from
 * a file in a given supported format and generating an ArrayList of Aircraft from it
 * for the optimization to work on.
 * 
 * Currently supported input types include:
 * - Plain text
 * 
 * @author William Hawkins
 *
 */
public class Input {

	private FCFSReader getReader(String type, String filename) throws FileNotFoundException, InvalidFCFSReaderTypeException {
		FCFSReaderType ftype = FCFSReaderType.valueOf(type.toUpperCase());
		if (FCFSReaderType.TEXT.equals(ftype)) {
			return new TextFCFSReader(filename);
		}
		throw new InvalidFCFSReaderTypeException("The FCFS reader type '" + type.toUpperCase() + "' does not exist.");
	}
	
	public ArrayList<Aircraft> generateFcfsSequence(String type, String filename) throws FileNotFoundException, InvalidFCFSReaderTypeException {
		
		FCFSReader reader = getReader(type, filename);
		
		// Read in the Aircraft in the order they appear in the file
		ArrayList<Aircraft> fcfs = new ArrayList<Aircraft>();
		Aircraft a;
		while ((a = reader.nextAircraft()) != null) {
			fcfs.add(a);
		}
		
		// Sort the aircraft by spot ready time, then set their 1-based FCFS indexes
		Collections.sort(fcfs, new AircraftSpotReadyComparator());
		int fcfsIndex = 1;
		for (Aircraft ac : fcfs) {
			ac.setFcfsIndex(fcfsIndex);
			fcfsIndex ++;
		}
		
		return fcfs;
	}
}
