package edu.mit.ll.group43.surfaceoptimization.network.dp;

import java.util.ArrayList;
import java.util.List;

/**
 * Represents a subsequence contained within a network node. Mostly used
 * in an attempt to order how nodes are set in a network stage to minimize
 * the number of unnecessary shifts that are made in an optimal sequence.
 * 
 * @author William Hawkins
 *
 * @param <T>
 */
@SuppressWarnings("serial")
public class Subsequence<T extends Comparable<T>> extends ArrayList<T> implements Comparable<Subsequence<T>> {
	
	public Subsequence() {
		super();
	}
	
	public Subsequence(List<T> init) {
		super(init);
	}
	
	/**
	 * Only allows elements that are not already in the Subsequence
	 * to be added to it.
	 * 
	 */
	@Override
	public boolean add(T element) {
		if (this.contains(element)) {
			return false;
		}
		return super.add(element);
	}
	
	/**
	 * Returns the last element of this sequence.
	 * 
	 * @return
	 */
	public T last() {
		return this.get(this.size()-1);
	}
	
	@Override
	public int compareTo(Subsequence<T> s) {
		for (int i = 0; i < this.size(); i ++) {
			if (!this.get(i).equals(s.get(i))) {
				return this.get(i).compareTo(s.get(i));
			}
		}
		return 0;
	}
	
	@Override
	public String toString() {
		String str = "(";
		str += this.get(0).toString();
		for (int i = 1; i < this.size(); i ++) {
			str += "-" + this.get(i).toString();
		}
		str += ")";
		return str;
	}

}
