package edu.mit.ll.group43.surfaceoptimization.network.dp;

import java.util.ArrayList;
import java.util.List;
import java.util.TreeSet;

import edu.mit.ll.group43.surfaceoptimization.network.NetworkEdge;
import edu.mit.ll.group43.surfaceoptimization.network.NetworkNode;

/**
 * Represents a node in a network created from a table of valid
 * locations for various indexes from an FCFS sequence, with respect
 * to a CPS value. Such a network is used to generate the subsequences
 * stored in the nodes of the CPS network.
 * 
 * @author William Hawkins
 *
 */
public class SubsequenceGeneratorNode implements NetworkNode {

	private ArrayList<NetworkEdge> prevNodes;
	private ArrayList<NetworkEdge> nextNodes;
	
	private int fcfs;
	private TreeSet<Subsequence<Integer>> validPaths;
	
	public SubsequenceGeneratorNode(int fcfs) {
		this.fcfs = fcfs;
		this.prevNodes = new ArrayList<NetworkEdge>();
		this.nextNodes = new ArrayList<NetworkEdge>();
		this.validPaths = new TreeSet<Subsequence<Integer>>();
	}
	
	@Override
	public ArrayList<NetworkEdge> getNextNodes() {
		return nextNodes;
	}

	@Override
	public ArrayList<NetworkEdge> getPrevNodes() {
		return prevNodes;
	}

	@Override
	public void setNextNodes(ArrayList<NetworkEdge> nextNodes) {
		this.nextNodes = nextNodes;
	}
	
	public void addPrevNode(NetworkEdge prevNode) {
		this.prevNodes.add(prevNode);
	}
	
	public void addNextNode(NetworkEdge nextNode) {
		this.nextNodes.add(nextNode);
	}

	@Override
	public void setPrevNodes(ArrayList<NetworkEdge> prevNodes) {
		this.prevNodes = prevNodes;
	}
	
	public int getFcfs() {
		return fcfs;
	}
	
	public TreeSet<Subsequence<Integer>> getValidPaths() {
		return validPaths;
	}

	public void setValidPaths(TreeSet<Subsequence<Integer>> validPaths) {
		this.validPaths = validPaths;
	}
	
	public void addValidPath(Subsequence<Integer> path, int cps) {
		
		// Calculate the to (inclusive) and from (exclusive) indices to look at,
		// and then get the sub-list
		int begin = Math.max(0, path.size() - 2 * cps);
		int end = path.size();
		List<Integer> subList = path.subList(begin, end);
		
		// Only try to add the new path if this index is not already in the sub-list
		// and the newly created sub-list does not already exist
		Subsequence<Integer> newPath = new Subsequence<Integer>(subList);
		if (newPath.add(this.fcfs) && !this.validPaths.contains(newPath)) {
			this.validPaths.add(newPath);
		}
	}
}
