package edu.mit.ll.group43.surfaceoptimization.output;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;
import edu.mit.ll.group43.surfaceoptimization.aircraft.AircraftAlphaComparator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.AircraftSpotReadyComparator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.DefaultWeightClass;

/**
 * Outputs MATLAB code that can be entered into the terminal and run,
 * provided that the MATLAB code for stochastic testing is available.
 * 
 * @author William Hawkins
 *
 */
public class MatlabStochasticCodeOutput implements Output {

	PrintWriter writer;
	
	@Override
	public void output(OutputStream os, ArrayList<Aircraft> sequence) {
		
		writer = new PrintWriter(os, true);
		
		setOptTakeOffTimes(sequence);
		
		Collections.sort(sequence, Collections.reverseOrder(new AircraftSpotReadyComparator()));
		writer.print("A=");
		outputMATLABData(getAircraftTypes(sequence));
		writer.print("B=");
		outputMATLABData(getSpotReadyTimes(sequence));
		writer.print("D=");
		outputMATLABData(getUnimpTaxiTimes(sequence));
		writer.print("E=");
		outputMATLABData(getOrigRunwayReadyTimes(sequence));
		writer.print("F=");
		outputMATLABData(getOptTakeOffTimes(sequence));
		writer.print("G=");
		outputMATLABData(getOptTakeOffSeqIndexes(sequence));
		writer.print("J=");
		outputMATLABData(getOptSpotLeaveTimes(sequence));
		writer.println("Stochasticity(A,B,D,E,F,G,J)");

	}
	
	public void outputMATLABData(int[] data) {
		writer.print("[");
		for (int i = 0; i < data.length-1; i ++) {
			writer.print(data[i] + "; ");
		}
		writer.println(data[data.length-1] + "];");
	}
	
	public void setOptTakeOffTimes(ArrayList<Aircraft> sequence) {
		Collections.sort(sequence, AircraftAlphaComparator.getInstance());
		for (int i = 0; i < sequence.size(); i ++) {
			sequence.get(i).setOptTakeOffIndex(i + 1);
		}
	}
	
	public int[] getAircraftTypes(ArrayList<Aircraft> sequence) {
		int[] acTypes = new int[sequence.size()];
		for (int i = 0; i < acTypes.length; i ++) {
			acTypes[i] = Arrays.binarySearch(DefaultWeightClass.values(), 
					sequence.get(i).getWeightClass()) + 1;
		}
		return acTypes;
	}
	
	public int[] getSpotReadyTimes(ArrayList<Aircraft> sequence) {
		int[] acTypes = new int[sequence.size()];
		for (int i = 0; i < acTypes.length; i ++) {
			acTypes[i] = sequence.get(i).getSpotReadyTime();
		}
		return acTypes;
	}
	
	public int[] getUnimpTaxiTimes(ArrayList<Aircraft> sequence) {
		int[] acTypes = new int[sequence.size()];
		for (int i = 0; i < acTypes.length; i ++) {
			acTypes[i] = sequence.get(i).getUnimpTaxiTime();
		}
		return acTypes;
	}
	
	public int[] getOrigRunwayReadyTimes(ArrayList<Aircraft> sequence) {
		int[] acTypes = new int[sequence.size()];
		for (int i = 0; i < acTypes.length; i ++) {
			acTypes[i] = sequence.get(i).getSpotReadyTime() + sequence.get(i).getUnimpTaxiTime();
		}
		return acTypes;
	}
	
	public int[] getOptTakeOffTimes(ArrayList<Aircraft> sequence) {
		int[] acTypes = new int[sequence.size()];
		for (int i = 0; i < acTypes.length; i ++) {
			acTypes[i] = sequence.get(i).getRunwayReadyTime();
		}
		return acTypes;
	}
	
	public int[] getOptTakeOffSeqIndexes(ArrayList<Aircraft> sequence) {
		int[] acTypes = new int[sequence.size()];
		for (int i = 0; i < acTypes.length; i ++) {
			acTypes[i] = sequence.get(i).getOptTakeOffIndex();
		}
		return acTypes;
	}
	
	public int[] getOptSpotLeaveTimes(ArrayList<Aircraft> sequence) {
		int[] acTypes = new int[sequence.size()];
		for (int i = 0; i < acTypes.length; i ++) {
			acTypes[i] = sequence.get(i).getOptReleaseTime();
		}
		return acTypes;
	}

}
